/**
 *
 *                                  Apache License
 *                            Version 2.0, January 2004
 *                         http://www.apache.org/licenses/
 *
 *    TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
 *
 *    1. Definitions.
 *
 *       "License" shall mean the terms and conditions for use, reproduction,
 *       and distribution as defined by Sections 1 through 9 of this document.
 *
 *       "Licensor" shall mean the copyright owner or entity authorized by
 *       the copyright owner that is granting the License.
 *
 *       "Legal Entity" shall mean the union of the acting entity and all
 *       other entities that control, are controlled by, or are under common
 *       control with that entity. For the purposes of this definition,
 *       "control" means (i) the power, direct or indirect, to cause the
 *       direction or management of such entity, whether by contract or
 *       otherwise, or (ii) ownership of fifty percent (50%) or more of the
 *       outstanding shares, or (iii) beneficial ownership of such entity.
 *
 *       "You" (or "Your") shall mean an individual or Legal Entity
 *       exercising permissions granted by this License.
 *
 *       "Source" form shall mean the preferred form for making modifications,
 *       including but not limited to software source code, documentation
 *       source, and configuration files.
 *
 *       "Object" form shall mean any form resulting from mechanical
 *       transformation or translation of a Source form, including but
 *       not limited to compiled object code, generated documentation,
 *       and conversions to other media types.
 *
 *       "Work" shall mean the work of authorship, whether in Source or
 *       Object form, made available under the License, as indicated by a
 *       copyright notice that is included in or attached to the work
 *       (an example is provided in the Appendix below).
 *
 *       "Derivative Works" shall mean any work, whether in Source or Object
 *       form, that is based on (or derived from) the Work and for which the
 *       editorial revisions, annotations, elaborations, or other modifications
 *       represent, as a whole, an original work of authorship. For the purposes
 *       of this License, Derivative Works shall not include works that remain
 *       separable from, or merely link (or bind by name) to the interfaces of,
 *       the Work and Derivative Works thereof.
 *
 *       "Contribution" shall mean any work of authorship, including
 *       the original version of the Work and any modifications or additions
 *       to that Work or Derivative Works thereof, that is intentionally
 *       submitted to Licensor for inclusion in the Work by the copyright owner
 *       or by an individual or Legal Entity authorized to submit on behalf of
 *       the copyright owner. For the purposes of this definition, "submitted"
 *       means any form of electronic, verbal, or written communication sent
 *       to the Licensor or its representatives, including but not limited to
 *       communication on electronic mailing lists, source code control systems,
 *       and issue tracking systems that are managed by, or on behalf of, the
 *       Licensor for the purpose of discussing and improving the Work, but
 *       excluding communication that is conspicuously marked or otherwise
 *       designated in writing by the copyright owner as "Not a Contribution."
 *
 *       "Contributor" shall mean Licensor and any individual or Legal Entity
 *       on behalf of whom a Contribution has been received by Licensor and
 *       subsequently incorporated within the Work.
 *
 *    2. Grant of Copyright License. Subject to the terms and conditions of
 *       this License, each Contributor hereby grants to You a perpetual,
 *       worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *       copyright license to reproduce, prepare Derivative Works of,
 *       publicly display, publicly perform, sublicense, and distribute the
 *       Work and such Derivative Works in Source or Object form.
 *
 *    3. Grant of Patent License. Subject to the terms and conditions of
 *       this License, each Contributor hereby grants to You a perpetual,
 *       worldwide, non-exclusive, no-charge, royalty-free, irrevocable
 *       (except as stated in this section) patent license to make, have made,
 *       use, offer to sell, sell, import, and otherwise transfer the Work,
 *       where such license applies only to those patent claims licensable
 *       by such Contributor that are necessarily infringed by their
 *       Contribution(s) alone or by combination of their Contribution(s)
 *       with the Work to which such Contribution(s) was submitted. If You
 *       institute patent litigation against any entity (including a
 *       cross-claim or counterclaim in a lawsuit) alleging that the Work
 *       or a Contribution incorporated within the Work constitutes direct
 *       or contributory patent infringement, then any patent licenses
 *       granted to You under this License for that Work shall terminate
 *       as of the date such litigation is filed.
 *
 *    4. Redistribution. You may reproduce and distribute copies of the
 *       Work or Derivative Works thereof in any medium, with or without
 *       modifications, and in Source or Object form, provided that You
 *       meet the following conditions:
 *
 *       (a) You must give any other recipients of the Work or
 *           Derivative Works a copy of this License; and
 *
 *       (b) You must cause any modified files to carry prominent notices
 *           stating that You changed the files; and
 *
 *       (c) You must retain, in the Source form of any Derivative Works
 *           that You distribute, all copyright, patent, trademark, and
 *           attribution notices from the Source form of the Work,
 *           excluding those notices that do not pertain to any part of
 *           the Derivative Works; and
 *
 *       (d) If the Work includes a "NOTICE" text file as part of its
 *           distribution, then any Derivative Works that You distribute must
 *           include a readable copy of the attribution notices contained
 *           within such NOTICE file, excluding those notices that do not
 *           pertain to any part of the Derivative Works, in at least one
 *           of the following places: within a NOTICE text file distributed
 *           as part of the Derivative Works; within the Source form or
 *           documentation, if provided along with the Derivative Works; or,
 *           within a display generated by the Derivative Works, if and
 *           wherever such third-party notices normally appear. The contents
 *           of the NOTICE file are for informational purposes only and
 *           do not modify the License. You may add Your own attribution
 *           notices within Derivative Works that You distribute, alongside
 *           or as an addendum to the NOTICE text from the Work, provided
 *           that such additional attribution notices cannot be construed
 *           as modifying the License.
 *
 *       You may add Your own copyright statement to Your modifications and
 *       may provide additional or different license terms and conditions
 *       for use, reproduction, or distribution of Your modifications, or
 *       for any such Derivative Works as a whole, provided Your use,
 *       reproduction, and distribution of the Work otherwise complies with
 *       the conditions stated in this License.
 *
 *    5. Submission of Contributions. Unless You explicitly state otherwise,
 *       any Contribution intentionally submitted for inclusion in the Work
 *       by You to the Licensor shall be under the terms and conditions of
 *       this License, without any additional terms or conditions.
 *       Notwithstanding the above, nothing herein shall supersede or modify
 *       the terms of any separate license agreement you may have executed
 *       with Licensor regarding such Contributions.
 *
 *    6. Trademarks. This License does not grant permission to use the trade
 *       names, trademarks, service marks, or product names of the Licensor,
 *       except as required for reasonable and customary use in describing the
 *       origin of the Work and reproducing the content of the NOTICE file.
 *
 *    7. Disclaimer of Warranty. Unless required by applicable law or
 *       agreed to in writing, Licensor provides the Work (and each
 *       Contributor provides its Contributions) on an "AS IS" BASIS,
 *       WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 *       implied, including, without limitation, any warranties or conditions
 *       of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A
 *       PARTICULAR PURPOSE. You are solely responsible for determining the
 *       appropriateness of using or redistributing the Work and assume any
 *       risks associated with Your exercise of permissions under this License.
 *
 *    8. Limitation of Liability. In no event and under no legal theory,
 *       whether in tort (including negligence), contract, or otherwise,
 *       unless required by applicable law (such as deliberate and grossly
 *       negligent acts) or agreed to in writing, shall any Contributor be
 *       liable to You for damages, including any direct, indirect, special,
 *       incidental, or consequential damages of any character arising as a
 *       result of this License or out of the use or inability to use the
 *       Work (including but not limited to damages for loss of goodwill,
 *       work stoppage, computer failure or malfunction, or any and all
 *       other commercial damages or losses), even if such Contributor
 *       has been advised of the possibility of such damages.
 *
 *    9. Accepting Warranty or Additional Liability. While redistributing
 *       the Work or Derivative Works thereof, You may choose to offer,
 *       and charge a fee for, acceptance of support, warranty, indemnity,
 *       or other liability obligations and/or rights consistent with this
 *       License. However, in accepting such obligations, You may act only
 *       on Your own behalf and on Your sole responsibility, not on behalf
 *       of any other Contributor, and only if You agree to indemnify,
 *       defend, and hold each Contributor harmless for any liability
 *       incurred by, or claims asserted against, such Contributor by reason
 *       of your accepting any such warranty or additional liability.
 *
 *    END OF TERMS AND CONDITIONS
 *
 *    APPENDIX: How to apply the Apache License to your work.
 *
 *       To apply the Apache License to your work, attach the following
 *       boilerplate notice, with the fields enclosed by brackets "[]"
 *       replaced with your own identifying information. (Don't include
 *       the brackets!)  The text should be enclosed in the appropriate
 *       comment syntax for the file format. We also recommend that a
 *       file or class name and description of purpose be included on the
 *       same "printed page" as the copyright notice for easier
 *       identification within third-party archives.
 *
 *    Copyright 2016 Alibaba Group
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.alibaba.weex.extend.module.location;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageManager;
import android.location.Address;
import android.location.Criteria;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v4.app.ActivityCompat;
import android.text.TextUtils;

import com.taobao.weex.WXEnvironment;
import com.taobao.weex.WXSDKInstance;
import com.taobao.weex.WXSDKManager;
import com.taobao.weex.utils.WXLogUtils;

import org.json.JSONException;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by lixinke on 16/9/10.
 */
public class DefaultLocation implements ILocatable {

  private static final String TAG = "DefaultLocation";
  private static final int TIME_OUT_WHAT = 0x11;

  private Map<String, WXLocationListener> mRegisterSucCallbacks = new HashMap<>();
  private List<WXLocationListener> mWXLocationListeners = new ArrayList<>();
  private WXSDKInstance mWXSDKInstance;
  private LocationManager mLocationManager;

  private static final int GPS_TIMEOUT = 15000;
  private int MIN_TIME = 20000;
  private int MIN_DISTANCE = 5;

  public DefaultLocation(WXSDKInstance instance) {
    mWXSDKInstance = instance;
  }

  @Override
  public void setWXSDKInstance(WXSDKInstance WXSDKInstance) {
    mWXSDKInstance = WXSDKInstance;
  }

  @Override
  public void getCurrentPosition(final String successCallback, final String errorCallback, final String params) {
    if(WXEnvironment.isApkDebugable()) {
      WXLogUtils.d(TAG, "into--[getCurrentPosition] successCallback:" + successCallback + " \nerrorCallback:" + errorCallback + " \nparams:" + params);
    }
    if (!TextUtils.isEmpty(params)) {
      try {
        org.json.JSONObject jsObj = new org.json.JSONObject(params);
        boolean enableHighAccuracy = jsObj.optBoolean("enableHighAccuracy");
        boolean enableAddress = jsObj.optBoolean("address");
        WXLocationListener listener = findLocation(null, successCallback, errorCallback, enableHighAccuracy, enableAddress);
        if (listener != null) {
          mWXLocationListeners.add(listener);
        }
        return;
      } catch (JSONException e) {
        WXLogUtils.e(TAG, e);
      }
    }
    Map<String, Object> options = new HashMap<>();
    options.put(ERROR_CODE, ErrorCode.PARAMS_ERROR);
    options.put(ERROR_MSG, ErrorMsg.PARAMS_ERROR);
    WXSDKManager.getInstance().callback(mWXSDKInstance.getInstanceId(), errorCallback, options);
  }

  private WXLocationListener findLocation(String watchId, String sucCallback, String errorCallback, boolean enableHighAccuracy, boolean enableAddress) {
//    WXLogUtils.d(TAG, "into--[findLocation] mWatchId:" + watchId + "\nsuccessCallback:" + sucCallback + "\nerrorCallback:" + errorCallback + "\nenableHighAccuracy:" + enableHighAccuracy + "\nmEnableAddress:" + enableAddress);

    if (mLocationManager == null) {
      mLocationManager = (LocationManager) mWXSDKInstance.getContext().getSystemService(Context.LOCATION_SERVICE);
    }
    Criteria criteria = new Criteria();
    if (enableHighAccuracy) {
      criteria.setAccuracy(Criteria.ACCURACY_COARSE);
    }
    //String provider = locationManager.getBestProvider(criteria, false);
    if (ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
      WXLocationListener WXLocationListener = new WXLocationListener(mLocationManager, mWXSDKInstance, watchId, sucCallback, errorCallback, enableAddress);
      mLocationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, MIN_TIME, MIN_DISTANCE, WXLocationListener);
      mLocationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, MIN_TIME, MIN_DISTANCE, WXLocationListener);
      return WXLocationListener;
    } else {
      Map<String, Object> options = new HashMap<>();
      options.put(ERROR_CODE, ErrorCode.NO_PERMISSION_ERROR);
      options.put(ERROR_MSG, ErrorMsg.NO_PERMISSION_ERROR);
      WXSDKManager.getInstance().callback(mWXSDKInstance.getInstanceId(), errorCallback, options);
    }
    return null;
  }

  @Override
  public void watchPosition(final String successCallback, final String errorCallback, final String params) {
    WXLogUtils.d("into--[watchPosition] successCallback:" + successCallback + " errorCallback:" + errorCallback + "\nparams:" + params);
    if (!TextUtils.isEmpty(params)) {
      try {
        org.json.JSONObject jsObj = new org.json.JSONObject(params);
        boolean enableHighAccuracy = jsObj.optBoolean("enableHighAccuracy");
        boolean enableAddress = jsObj.optBoolean("address");

        String id = UUID.randomUUID().toString();
        WXLocationListener listener = findLocation(id, successCallback, errorCallback, enableHighAccuracy, enableAddress);
        if (listener != null) {
          mRegisterSucCallbacks.put(id, listener);
        }
        return;
      } catch (JSONException e) {
        WXLogUtils.e(TAG, e);
      }
    }
    Map<String, Object> options = new HashMap<>();
    options.put(ERROR_CODE, ErrorCode.PARAMS_ERROR);
    options.put(ERROR_MSG, ErrorMsg.PARAMS_ERROR);
    WXSDKManager.getInstance().callback(mWXSDKInstance.getInstanceId(), errorCallback, options);
  }

  @Override
  public void clearWatch(String watchId) {
    WXLogUtils.d("into--[clearWatch] mWatchId:" + watchId);
    if (mWXSDKInstance == null || mWXSDKInstance.isDestroy() || mLocationManager == null) {
      return;
    }
    if (ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
      WXLocationListener listener = mRegisterSucCallbacks.get(watchId);
      if (listener != null) {
        listener.destroy();
        mLocationManager.removeUpdates(listener);
      }
      mRegisterSucCallbacks.remove(watchId);
    }
  }

  @Override
  public void destroy() {
    WXLogUtils.d("into--[destroy]");
    if (mWXSDKInstance == null || mWXSDKInstance.isDestroy() || mLocationManager == null) {
      return;
    }

    if (mWXLocationListeners != null && mWXLocationListeners.size() > 0) {
      if (ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
        for (WXLocationListener listener : mWXLocationListeners) {
          if (listener != null) {
            listener.destroy();
            mLocationManager.removeUpdates(listener);
          }
        }
        mWXLocationListeners.clear();
      }
    }

    if (mRegisterSucCallbacks != null && mRegisterSucCallbacks.size() > 0) {
      Collection<WXLocationListener> values = mRegisterSucCallbacks.values();
      if (ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
        for (WXLocationListener listener : values) {
          listener.destroy();
          mLocationManager.removeUpdates(listener);
        }
        mRegisterSucCallbacks.clear();
      }
    }
  }

  static class WXLocationListener implements LocationListener, Handler.Callback {

    private WXSDKInstance mWXSDKInstance;
    private String mWatchId;
    private String mSucCallback;
    private String mErrorCallback;
    private boolean mEnableAddress;
    private Handler mHandler;
    private LocationManager mLocationManager;

    private WXLocationListener(LocationManager locationManager, WXSDKInstance instance, String watchId, String sucCallback, String errorCallback, boolean enableAddress) {
      mWXSDKInstance = instance;
      this.mWatchId = watchId;
      this.mSucCallback = sucCallback;
      this.mErrorCallback = errorCallback;
      this.mEnableAddress = enableAddress;
      mHandler = new Handler(this);
      mLocationManager = locationManager;
//      WVThreadPool.getInstance().execute(new Runnable() {
//        public void run() {
          mHandler.sendEmptyMessageDelayed(TIME_OUT_WHAT, GPS_TIMEOUT);
//        }
//      });
    }

    @Override
    public void onLocationChanged(Location location) {
      mHandler.removeMessages(TIME_OUT_WHAT);
      if(WXEnvironment.isApkDebugable()) {
        WXLogUtils.d(TAG, "into--[onLocationChanged] location:" + location == null ? "Location is NULL!" : location.toString());
      }

      if (mWXSDKInstance == null || mWXSDKInstance.isDestroy()) {
        return;
      }

      if (location != null) {
        Map<String, Object> result = new HashMap<>();
        Map<String, Object> coords = new HashMap<>();
        double longitude = location.getLongitude();
        double latitude = location.getLatitude();
        coords.put("longitude", longitude);
        coords.put("latitude", latitude);
        coords.put("altitude", location.getAltitude());
        coords.put("accuracy", location.getAccuracy());
        coords.put("heading", location.getBearing());
        coords.put("speed", location.getSpeed());
        result.put(COORDS, coords);

        // 行政区域
        if (mEnableAddress) {
          Address addr = getAddress(latitude, longitude);
          Map<String, Object> address = new HashMap<>();
          if (addr != null) {
            address.put("country", addr.getCountryName());
            address.put("province", addr.getAdminArea());
            address.put("city", addr.getLocality());
            address.put("cityCode", addr.getPostalCode());
            address.put("area", addr.getSubLocality());
            address.put("road", addr.getThoroughfare());
            // 其中getAddressLine(0)表示国家，getAddressLine(1)表示精确到某个区，getAddressLine(2)表示精确到具体的街
            StringBuilder sb = new StringBuilder();
            for (int i = 1; i <= 2; i++) {
              if (!TextUtils.isEmpty(addr.getAddressLine(i))) { //防止null导致的拼接错误
                sb.append(addr.getAddressLine(i));
              }
            }
            address.put("addressLine", sb.toString());
          }
          result.put(ADDRESS, address);
        }
        result.put(ERROR_CODE, ErrorCode.SUCCESS);
        result.put(ERROR_MSG, ErrorMsg.SUCCESS);
        if (!TextUtils.isEmpty(mWatchId)) {
          result.put(WATCH_ID, mWatchId);
        }
        WXSDKManager.getInstance().callback(mWXSDKInstance.getInstanceId(), mSucCallback, result, TextUtils.isEmpty(mWatchId) ? false : true);
      } else {
        Map<String, Object> options = new HashMap<>();
        options.put(ERROR_CODE, ErrorCode.LOCATION_ERROR);
        options.put(ERROR_MSG, ErrorMsg.LOCATION_ERROR);
        if (!TextUtils.isEmpty(mWatchId)) {
          options.put(WATCH_ID, mWatchId);
        }
        WXSDKManager.getInstance().callback(mWXSDKInstance.getInstanceId(), mErrorCallback, options, TextUtils.isEmpty(mWatchId) ? false : true);
      }
      if (TextUtils.isEmpty(mWatchId)) {
        if (mLocationManager != null && ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
          destroy();
          mLocationManager.removeUpdates(this);
        }
      }
    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {
      WXLogUtils.i(TAG, "into--[onStatusChanged] provider:" + provider + " status:" + status);
    }

    @Override
    public void onProviderEnabled(String provider) {
      WXLogUtils.i(TAG, "into--[onProviderEnabled] provider:" + provider);
    }

    @Override
    public void onProviderDisabled(String provider) {
      WXLogUtils.i(TAG, "into--[onProviderDisabled] provider:" + provider);
    }

    @Override
    public boolean handleMessage(Message msg) {
      if (msg.what == TIME_OUT_WHAT) {
        if(WXEnvironment.isApkDebugable()) {
          WXLogUtils.d(TAG, "into--[handleMessage] Location Time Out!");
        }
        if (mWXSDKInstance == null || mWXSDKInstance.isDestroy() || mLocationManager == null) {
          return false;
        }
        if (ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(mWXSDKInstance.getContext(), Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
          mLocationManager.removeUpdates(this);
        }
        Map<String, Object> options = new HashMap<>();
        options.put(ERROR_CODE, ErrorCode.LOCATION_TIME_OUT);
        options.put(ERROR_MSG, ErrorMsg.LOCATION_TIME_OUT);
        if (!TextUtils.isEmpty(mWatchId)) {
          options.put(WATCH_ID, mWatchId);
        }
        WXSDKManager.getInstance().callback(mWXSDKInstance.getInstanceId(), mErrorCallback, options);
        return true;
      }
      return false;
    }

    /**
     * get address info
     */
    private Address getAddress(double latitude, double longitude) {
      if(WXEnvironment.isApkDebugable()) {
        WXLogUtils.d(TAG, "into--[getAddress] latitude:" + latitude + " longitude:" + longitude);
      }
      try {
        if (mWXSDKInstance == null || mWXSDKInstance.isDestroy()) {
          return null;
        }
        Geocoder gc = new Geocoder(mWXSDKInstance.getContext());
        List<Address> list = gc.getFromLocation(latitude, longitude, 1);
        if (list != null && list.size() > 0) {
          return list.get(0);
        }
      } catch (Exception e) {
        WXLogUtils.e(TAG, e);
      }
      return null;
    }

    public void destroy() {
      if (mHandler != null) {
        mHandler.removeMessages(TIME_OUT_WHAT);
        mWXSDKInstance=null;
      }
    }
  }
}